@@ -42,7 +42,7 @@
 #define MD_USER_TLS_MD5SHA1 (GCRY_MODULE_ID_USER+1)
 
 /* Maximum length of a digest.  */
-#define MAX_DIGEST_LEN 36
+#define MAX_DIGEST_LEN 64
 
 /* A large struct name "opt" to keep global flags */
 struct
@@ -201,6 +201,10 @@ typedef enum
 cache_mode_t;
 
 
+/* The type of a function to lookup a TTL by a keygrip.  */
+typedef int (*lookup_ttl_t)(const char *hexgrip);
+
+
 /*-- gpg-agent.c --*/
 void agent_exit (int rc) JNLIB_GCC_A_NR; /* Also implemented in other tools */
 const char *get_agent_socket_name (void);
@@ -212,7 +216,8 @@ void agent_sighup_action (void);
 
 /*-- command.c --*/
 gpg_error_t agent_inq_pinentry_launched (ctrl_t ctrl, unsigned long pid);
-gpg_error_t agent_write_status (ctrl_t ctrl, const char *keyword, ...);
+gpg_error_t agent_write_status (ctrl_t ctrl, const char *keyword, ...)
+     GNUPG_GCC_A_SENTINEL(0);
 void bump_key_eventcounter (void);
 void bump_card_eventcounter (void);
 void start_command_handler (ctrl_t, gnupg_fd_t, gnupg_fd_t);
@@ -228,11 +233,15 @@ gpg_error_t agent_key_from_file (ctrl_t ctrl,
                                  const unsigned char *grip,
                                  unsigned char **shadow_info,
                                  cache_mode_t cache_mode,
+                                 lookup_ttl_t lookup_ttl,
                                  gcry_sexp_t *result);
 gpg_error_t agent_public_key_from_file (ctrl_t ctrl, 
                                         const unsigned char *grip,
                                         gcry_sexp_t *result);
 int agent_key_available (const unsigned char *grip);
+gpg_error_t agent_key_info_from_file (ctrl_t ctrl, const unsigned char *grip,
+                                      int *r_keytype,
+                                      unsigned char **r_shadow_info);
 
 /*-- call-pinentry.c --*/
 void initialize_module_call_pinentry (void);
@@ -245,9 +254,9 @@ int agent_askpin (ctrl_t ctrl,
                   struct pin_entry_info_s *pininfo);
 int agent_get_passphrase (ctrl_t ctrl, char **retpass,
                           const char *desc, const char *prompt,
-                          const char *errtext);
+                          const char *errtext, int with_qualitybar);
 int agent_get_confirmation (ctrl_t ctrl, const char *desc, const char *ok,
-			    const char *cancel);
+			    const char *notokay, int with_cancel);
 int agent_show_message (ctrl_t ctrl, const char *desc, const char *ok_btn);
 int agent_popup_message_start (ctrl_t ctrl,
                                const char *desc, const char *ok_btn);
@@ -266,7 +275,7 @@ void agent_unlock_cache_entry (void **cache_id);
 /*-- pksign.c --*/
 int agent_pksign_do (ctrl_t ctrl, const char *desc_text,
 		     gcry_sexp_t *signature_sexp,
-                     cache_mode_t cache_mode);
+                     cache_mode_t cache_mode, lookup_ttl_t lookup_ttl);
 int agent_pksign (ctrl_t ctrl, const char *desc_text,
                   membuf_t *outbuf, cache_mode_t cache_mode);
 
@@ -294,11 +303,13 @@ int agent_shadow_key (const unsigned char *pubkey,
                       unsigned char **result);
 int agent_get_shadow_info (const unsigned char *shadowkey,
                            unsigned char const **shadow_info);
+gpg_error_t parse_shadow_info (const unsigned char *shadow_info, 
+                               char **r_hexsn, char **r_idstr);
 
 
 /*-- trustlist.c --*/
 void initialize_module_trustlist (void);
-gpg_error_t agent_istrusted (ctrl_t ctrl, const char *fpr);
+gpg_error_t agent_istrusted (ctrl_t ctrl, const char *fpr, int *r_disabled);
 gpg_error_t agent_listtrusted (void *assuan_context);
 gpg_error_t agent_marktrusted (ctrl_t ctrl, const char *name,
                                const char *fpr, int flag);
