@@ -1,5 +1,5 @@
 /* divert-scd.c - divert operations to the scdaemon 
- *	Copyright (C) 2002, 2003 Free Software Foundation, Inc.
+ *	Copyright (C) 2002, 2003, 2009 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -28,16 +28,14 @@
 #include <sys/stat.h>
 
 #include "agent.h"
-#include "sexp-parse.h"
 #include "i18n.h"
+#include "sexp-parse.h"
 
 
 static int
 ask_for_card (ctrl_t ctrl, const unsigned char *shadow_info, char **r_kid)
 {
   int rc, i;
-  const unsigned char *s;
-  size_t n;
   char *serialno;
   int no_card = 0;
   char *desc;
@@ -45,39 +43,19 @@ ask_for_card (ctrl_t ctrl, const unsigned char *shadow_info, char **r_kid)
   int want_sn_displen;
 
   *r_kid = NULL;
-  s = shadow_info;
-  if (*s != '(')
-    return gpg_error (GPG_ERR_INV_SEXP);
-  s++;
-  n = snext (&s);
-  if (!n)
-    return gpg_error (GPG_ERR_INV_SEXP);
-  want_sn = xtrymalloc (n*2+1);
-  if (!want_sn)
-    return out_of_core ();
-  for (i=0; i < n; i++)
-    sprintf (want_sn+2*i, "%02X", s[i]);
-  s += n;
+
+  rc = parse_shadow_info (shadow_info, &want_sn, &want_kid);
+  if (rc)
+    return rc;
+
   /* We assume that a 20 byte serial number is a standard one which
-     seems to have the property to have a zero in the last nibble.  We
-     don't display this '0' because it may confuse the user */
+     has the property to have a zero in the last nibble (Due to BCD
+     representation).  We don't display this '0' because it may
+     confuse the user.  */
   want_sn_displen = strlen (want_sn);
   if (want_sn_displen == 20 && want_sn[19] == '0')
     want_sn_displen--;
 
-  n = snext (&s);
-  if (!n)
-    return gpg_error (GPG_ERR_INV_SEXP);
-  want_kid = xtrymalloc (n+1);
-  if (!want_kid)
-    {
-      gpg_error_t tmperr = out_of_core ();
-      xfree (want_sn);
-      return tmperr;
-    }
-  memcpy (want_kid, s, n);
-  want_kid[n] = 0;
-
   for (;;)
     {
       rc = agent_card_serialno (ctrl, &serialno);
@@ -119,7 +97,7 @@ ask_for_card (ctrl_t ctrl, const unsigned char *shadow_info, char **r_kid)
             }
           else
             {
-              rc = agent_get_confirmation (ctrl, desc, NULL, NULL);
+              rc = agent_get_confirmation (ctrl, desc, NULL, NULL, 0);
               xfree (desc);
             }
         }
@@ -181,10 +159,11 @@ encode_md_for_card (const unsigned char *digest, size_t digestlen, int algo,
 
    Flags:
 
-      'N' = New PIN, this requests a second prompt to repeat the the
+      'N' = New PIN, this requests a second prompt to repeat the
             PIN.  If the PIN is not correctly repeated it starts from
             all over.
-      'A' = The PIN is an Admin PIN, SO-PIN, PUK or alike.
+      'A' = The PIN is an Admin PIN, SO-PIN or alike.
+      'P' = The PIN is a PUK (Personal Unblocking Key).
       'R' = The PIN is a Reset Code.
 
    Example:
@@ -204,6 +183,7 @@ getpin_cb (void *opaque, const char *info, char *buf, size_t maxbuf)
   int any_flags = 0;
   int newpin = 0;
   int resetcode = 0;
+  int is_puk = 0;
   const char *again_text = NULL;
   const char *prompt = "PIN";
 
@@ -217,6 +197,13 @@ getpin_cb (void *opaque, const char *info, char *buf, size_t maxbuf)
         {
           if (*s == 'A')
             prompt = _("Admin PIN");
+          else if (*s == 'P')
+            {
+              /* TRANSLATORS: A PUK is the Personal Unblocking Code
+                 used to unblock a PIN. */
+              prompt = _("PUK");
+              is_puk = 1;
+            }
           else if (*s == 'N')
             newpin = 1;
           else if (*s == 'R')
@@ -242,7 +229,22 @@ getpin_cb (void *opaque, const char *info, char *buf, size_t maxbuf)
         }
       else if (maxbuf == 1)  /* Open the pinentry. */
         {
-          rc = agent_popup_message_start (ctrl, info, NULL);
+          if (info)
+            {
+              char *desc;
+
+              if ( asprintf (&desc,
+                             _("%s%%0A%%0AUse the reader's keypad for input."),
+                             info) < 0 )
+                rc = gpg_error_from_syserror ();
+              else
+                {
+                  rc = agent_popup_message_start (ctrl, desc, NULL);
+                  xfree (desc);
+                }
+            }
+          else
+            rc = agent_popup_message_start (ctrl, NULL, NULL);
         }
       else
         rc = gpg_error (GPG_ERR_INV_VALUE);
@@ -258,7 +260,7 @@ getpin_cb (void *opaque, const char *info, char *buf, size_t maxbuf)
     return gpg_error_from_syserror ();
   pi->max_length = maxbuf-1;
   pi->min_digits = 0;  /* we want a real passphrase */
-  pi->max_digits = 8;
+  pi->max_digits = 16;
   pi->max_tries = 3;
 
   if (any_flags)
@@ -277,17 +279,21 @@ getpin_cb (void *opaque, const char *info, char *buf, size_t maxbuf)
             }
           pi2->max_length = maxbuf-1;
           pi2->min_digits = 0;
-          pi2->max_digits = 8;
+          pi2->max_digits = 16;
           pi2->max_tries = 1;
           rc = agent_askpin (ctrl,
                              (resetcode?
                               _("Repeat this Reset Code"):
+                              is_puk?
+                              _("Repeat this PUK"):
                               _("Repeat this PIN")),
                              prompt, NULL, pi2);
           if (!rc && strcmp (pi->pin, pi2->pin))
             {
               again_text = (resetcode? 
                             N_("Reset Code not correctly repeated; try again"):
+                            is_puk?
+                            N_("PUK not correctly repeated; try again"):
                             N_("PIN not correctly repeated; try again"));
               xfree (pi2);
               xfree (pi);
