@@ -116,57 +116,9 @@ start_agent (void)
 static char *
 unescape_status_string (const unsigned char *s)
 {
-  char *buffer, *d;
-
-  buffer = d = xtrymalloc (strlen (s)+1);
-  if (!buffer)
-    return NULL;
-  while (*s)
-    {
-      if (*s == '%' && s[1] && s[2])
-        { 
-          s++;
-          *d = xtoi_2 (s);
-          if (!*d)
-            *d = '\xff';
-          d++;
-          s += 2;
-        }
-      else if (*s == '+')
-        {
-          *d++ = ' ';
-          s++;
-        }
-      else
-        *d++ = *s++;
-    }
-  *d = 0; 
-  return buffer;
+  return percent_plus_unescape (s, 0xff);
 }
 
-/* Copy the text ATEXT into the buffer P and do plus '+' and percent
-   escaping.  Note that the provided buffer needs to be 3 times the
-   size of ATEXT plus 1.  Returns a pointer to the leading Nul in P. */
-static char *
-my_percent_plus_escape (char *p, const char *atext)
-{
-  const unsigned char *s;
-
-  for (s=atext; *s; s++)
-    {
-      if (*s < ' ' || *s == '+' || *s == '%')
-        {
-          snprintf (p, 4, "%%%02X", *s);
-          p += 3;
-        }
-      else if (*s == ' ')
-        *p++ = '+';
-      else
-        *p++ = *s;
-    }
-  *p = 0;
-  return p;
-}
 
 /* Take a 20 byte hexencoded string and put it into the the provided
    20 byte buffer FPR in binary format. */
@@ -373,7 +325,19 @@ learn_status_cb (void *opaque, const char *line)
       else if (no == 3)
         parm->cafpr3valid = unhexify_fpr (line, parm->cafpr3);
     }
-  
+  else if (keywordlen == 8 && !memcmp (keyword, "KEY-ATTR", keywordlen))
+    {
+      int keyno, algo, nbits;
+
+      sscanf (line, "%d %d %d", &keyno, &algo, &nbits);
+      keyno--;
+      if (keyno >= 0 && keyno < DIM (parm->key_attr))
+        {
+          parm->key_attr[keyno].algo = algo;
+          parm->key_attr[keyno].nbits = nbits;
+        }
+    }
+
   return 0;
 }
 
@@ -391,6 +355,9 @@ agent_learn (struct agent_card_info_s *info)
   rc = assuan_transact (agent_ctx, "LEARN --send",
                         dummy_data_cb, NULL, default_inq_cb, NULL,
                         learn_status_cb, info);
+  /* Also try to get the key attributes.  */
+  if (!rc)
+    agent_scd_getattr ("KEY-ATTR", info);
   
   return rc;
 }
@@ -521,7 +488,6 @@ agent_scd_writecert (const char *certidstr,
 }
 
 
-
 
 /* Handle a KEYDATA inquiry.  Note, we only send the data,
    assuan_transact takes care of flushing and writing the end */
@@ -572,7 +538,6 @@ agent_scd_writekey (int keyno, const char *serialno,
 }
 
 
-
 
 /* Status callback for the SCD GENKEY command. */
 static int
@@ -583,7 +548,6 @@ scd_genkey_cb (void *opaque, const char *line)
   int keywordlen;
   gpg_error_t rc;
 
-  log_debug ("got status line `%s'\n", line);
   for (keywordlen=0; *line && !spacep (line); line++, keywordlen++)
     ;
   while (spacep (line))
@@ -701,7 +665,7 @@ agent_scd_pksign (const char *serialno, int hashalgo,
   /* Send the serialno command to initialize the connection. We don't
      care about the data returned.  If the card has already been
      initialized, this is a very fast command.  We request the openpgp
-     card because that is waht we expect. */
+     card because that is what we expect. */
   rc = assuan_transact (agent_ctx, "SCD SERIALNO openpgp",
                         NULL, NULL, NULL, NULL, NULL, NULL);
   if (rc)
@@ -766,7 +730,7 @@ agent_scd_pkdecrypt (const char *serialno,
   /* Send the serialno command to initialize the connection. We don't
      care about the data returned.  If the card has already been
      initialized, this is a very fast command.  We request the openpgp
-     card because that is waht we expect. */
+     card because that is what we expect. */
   rc = assuan_transact (agent_ctx, "SCD SERIALNO openpgp",
                         NULL, NULL, NULL, NULL, NULL, NULL);
   if (rc)
@@ -799,6 +763,43 @@ agent_scd_pkdecrypt (const char *serialno,
 }
 
 
+
+/* Send a READCERT command to the SCdaemon. */
+int 
+agent_scd_readcert (const char *certidstr,
+                    void **r_buf, size_t *r_buflen)
+{
+  int rc;
+  char line[ASSUAN_LINELENGTH];
+  membuf_t data;
+  size_t len;
+
+  *r_buf = NULL;
+  rc = start_agent ();
+  if (rc)
+    return rc;
+
+  init_membuf (&data, 2048);
+
+  snprintf (line, DIM(line)-1, "SCD READCERT %s", certidstr);
+  line[DIM(line)-1] = 0;
+  rc = assuan_transact (agent_ctx, line,
+                        membuf_data_cb, &data,
+                        default_inq_cb, NULL, NULL, NULL);
+  if (rc)
+    {
+      xfree (get_membuf (&data, &len));
+      return rc;
+    }
+  *r_buf = get_membuf (&data, r_buflen);
+  if (!*r_buf)
+    return gpg_error (GPG_ERR_ENOMEM);
+
+  return 0;
+}
+
+
+
 /* Change the PIN of an OpenPGP card or reset the retry counter.
    CHVNO 1: Change the PIN
          2: For v1 cards: Same as 1.
@@ -874,11 +875,16 @@ agent_get_passphrase (const char *cache_id,
                       const char *err_msg,
                       const char *prompt,
                       const char *desc_msg,
+                      int repeat,
+                      int check,
                       char **r_passphrase)
 {
   int rc;
-  char *line, *p;
-  char cmd[] = "GET_PASSPHRASE --data -- ";
+  char line[ASSUAN_LINELENGTH];
+  char *arg1 = NULL;
+  char *arg2 = NULL;  
+  char *arg3 = NULL; 
+  char *arg4 = NULL;
   membuf_t data;
 
   *r_passphrase = NULL;
@@ -887,41 +893,38 @@ agent_get_passphrase (const char *cache_id,
   if (rc)
     return rc;
 
-  /* We allocate 3 times the needed space for the texts so that
-     there is enough space for escaping. */
-  line = xtrymalloc ( strlen (cmd) + 1
-                      + (cache_id? 3*strlen (cache_id): 1) + 1
-                      + (err_msg?  3*strlen (err_msg): 1) + 1
-                      + (prompt?   3*strlen (prompt): 1) + 1
-                      + (desc_msg? 3*strlen (desc_msg): 1) + 1
-                      + 1);
-  if (!line)
-    return gpg_error_from_syserror ();
+  /* Check that the gpg-agent understands the repeat option.  */
+  if (assuan_transact (agent_ctx, 
+                       "GETINFO cmd_has_option GET_PASSPHRASE repeat",
+                       NULL, NULL, NULL, NULL, NULL, NULL))
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
 
-  p = stpcpy (line, cmd);
   if (cache_id && *cache_id)
-    p = my_percent_plus_escape (p, cache_id);
-  else
-    *p++ = 'X';
-  *p++ = ' ';
-
+    if (!(arg1 = percent_plus_escape (cache_id)))
+      goto no_mem;
   if (err_msg && *err_msg)
-    p = my_percent_plus_escape (p, err_msg);
-  else
-    *p++ = 'X';
-  *p++ = ' ';
-
+    if (!(arg2 = percent_plus_escape (err_msg)))
+      goto no_mem;
   if (prompt && *prompt)
-    p = my_percent_plus_escape (p, prompt);
-  else
-    *p++ = 'X'; 
-  *p++ = ' ';
-
+    if (!(arg3 = percent_plus_escape (prompt)))
+      goto no_mem;
   if (desc_msg && *desc_msg)
-    p = my_percent_plus_escape (p, desc_msg);
-  else
-    *p++ = 'X';
-  *p = 0;
+    if (!(arg4 = percent_plus_escape (desc_msg)))
+      goto no_mem;
+
+  snprintf (line, DIM(line)-1, 
+            "GET_PASSPHRASE --data --repeat=%d%s -- %s %s %s %s", 
+            repeat, 
+            check? " --check --qualitybar":"",
+            arg1? arg1:"X",
+            arg2? arg2:"X",
+            arg3? arg3:"X",
+            arg4? arg4:"X");
+  line[DIM(line)-1] = 0;
+  xfree (arg1);
+  xfree (arg2);
+  xfree (arg3);
+  xfree (arg4);
 
   init_membuf_secure (&data, 64);
   rc = assuan_transact (agent_ctx, line, 
@@ -937,7 +940,13 @@ agent_get_passphrase (const char *cache_id,
       if (!*r_passphrase)
         rc = gpg_error_from_syserror ();
     }
-  xfree (line);
+  return rc;
+ no_mem:
+  rc = gpg_error_from_syserror ();
+  xfree (arg1);
+  xfree (arg2);
+  xfree (arg3);
+  xfree (arg4);
   return rc;
 }
 
