@@ -116,7 +116,7 @@ my_strusage( int level )
     case 1:
     case 40: p = _("Usage: gpgv [options] [files] (-h for help)");
       break;
-    case 41: p = _("Syntax: gpg [options] [files]\n"
+    case 41: p = _("Syntax: gpgv [options] [files]\n"
                    "Check signatures against known trusted keys\n");
 	break;
 
@@ -193,12 +193,13 @@ main( int argc, char **argv )
 
   if (opt.verbose > 1)
     set_packet_list_mode(1);
-  
-  if (!nrings)  /* no keyring given: use default one */
-    keydb_add_resource ("trustedkeys" EXTSEP_S "gpg", 0, 0);
+
+  /* Note: We open all keyrings in read-only mode (flag value: 8).  */
+  if (!nrings)  /* No keyring given: use default one. */
+    keydb_add_resource ("trustedkeys" EXTSEP_S "gpg", 8, 0);
   for (sl = nrings; sl; sl = sl->next)
-    keydb_add_resource (sl->d, 0, 0 );
-    
+    keydb_add_resource (sl->d, 8, 0 );
+   
   FREE_STRLIST (nrings);
     
   if ( (rc = verify_signatures( argc, argv ) ))
