@@ -1,6 +1,6 @@
 /* mainproc.c - handle packets
  * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
- *               2008 Free Software Foundation, Inc.
+ *               2008, 2009 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -233,7 +233,7 @@ symkey_decrypt_seskey( DEK *dek, byte *seskey, size_t slen )
       return G10ERR_BAD_KEY;
     }
 
-  if (gcry_cipher_open (&hd, dek->algo, GCRY_CIPHER_MODE_CFB, 1))
+  if (openpgp_cipher_open (&hd, dek->algo, GCRY_CIPHER_MODE_CFB, 1))
       BUG ();
   if (gcry_cipher_setkey ( hd, dek->key, dek->keylen ))
     BUG ();
@@ -311,7 +311,7 @@ proc_symkey_enc( CTX c, PACKET *pkt )
 	  }
 	else
 	  {
-	    c->dek = passphrase_to_dek (NULL, 0, algo, &enc->s2k, 0,
+	    c->dek = passphrase_to_dek (NULL, 0, algo, &enc->s2k, 3,
                                         NULL, NULL);
 	    if(c->dek)
 	      {
@@ -469,8 +469,8 @@ print_pkenc_list( struct kidlist_item *list, int failed )
 	if( list->reason == G10ERR_NO_SECKEY ) {
 	    if( is_status_enabled() ) {
 		char buf[20];
-		sprintf(buf,"%08lX%08lX", (ulong)list->kid[0],
-					  (ulong)list->kid[1] );
+		snprintf (buf, sizeof buf, "%08lX%08lX",
+                          (ulong)list->kid[0], (ulong)list->kid[1]);
 		write_status_text( STATUS_NO_SECKEY, buf );
 	    }
 	}
@@ -548,7 +548,7 @@ proc_encrypted( CTX c, PACKET *pkt )
 		log_info (_("assuming %s encrypted data\n"), "IDEA");
 	      }
 
-	    c->dek = passphrase_to_dek ( NULL, 0, algo, s2k, 0, NULL, NULL );
+	    c->dek = passphrase_to_dek ( NULL, 0, algo, s2k, 3, NULL, NULL );
 	    if (c->dek)
 	      c->dek->algo_info_printed = 1;
 	  }
