@@ -25,7 +25,7 @@
 #include <assert.h>
 
 #include "gpg.h"
-#include "errors.h"
+#include "status.h"
 #include "iobuf.h"
 #include "util.h"
 #include "filter.h"
