@@ -139,9 +139,19 @@ mpi_read (iobuf_t inp, unsigned int *ret_nread, int secure)
       p[i+2] = iobuf_get(inp) & 0xff;
       nread++;
     }
-  if ( gcry_mpi_scan( &a, GCRYMPI_FMT_PGP, buf, nread, &nread ) )
-    a = NULL;
-    
+
+  if (nread >= 2 && !(buf[0] << 8 | buf[1]))
+    {
+      /* Libgcrypt < 1.5.0 accidently rejects zero-length (i.e. zero)
+         MPIs.  We fix this here.  */
+      a = gcry_mpi_new (0);
+    }
+  else
+    {
+      if ( gcry_mpi_scan( &a, GCRYMPI_FMT_PGP, buf, nread, &nread ) )
+        a = NULL;
+    }
+
  leave:
   gcry_free(buf);
   if ( nread > *ret_nread )
@@ -491,8 +501,8 @@ parse( IOBUF inp, PACKET *pkt, int onlykeypkts, off_t *retpos,
 	  }
       }
 
-    if (pktlen == 0xffffffff) {
-        /* with a some probability this is caused by a problem in the
+    if (pktlen == (unsigned long)(-1)) {
+        /* With some probability this is caused by a problem in the
          * the uncompressing layer - in some error cases it just loops
          * and spits out 0xff bytes. */
         log_error ("%s: garbled packet detected\n", iobuf_where(inp) );
@@ -1672,6 +1682,7 @@ parse_key (IOBUF inp, int pkttype, unsigned long pktlen,
     int npkey, nskey;
     int is_v4=0;
     int rc=0;
+    u32 keyid[2];
 
     (void)hdr;
 
@@ -1997,6 +2008,9 @@ parse_key (IOBUF inp, int pkttype, unsigned long pktlen,
 		fprintf (listfp, "\tchecksum: %04hx\n", sk->csum);
 	    }
 	}
+
+        if (list_mode)
+          keyid_from_sk (sk, keyid);
     }
     else {
 	PKT_public_key *pk = pkt->pkt.public_key;
@@ -2021,8 +2035,14 @@ parse_key (IOBUF inp, int pkttype, unsigned long pktlen,
 	}
         if (rc)
             goto leave;
+        if (list_mode)
+          keyid_from_pk (pk, keyid);
     }
 
+    if (list_mode)
+      fprintf (listfp, "\tkeyid: %08lX%08lX\n", 
+               (ulong)keyid[0], (ulong)keyid[1]);
+
   leave:
     iobuf_skip_rest(inp, pktlen, 0);
     return rc;
