@@ -6,7 +6,7 @@
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -297,8 +295,9 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 	}
 	else { /* binary mode */
 	    byte *buffer = xmalloc( 32768 );
-	    int eof;
-	    for( eof=0; !eof; ) {
+	    int eof_seen = 0;
+
+	    while ( !eof_seen ) {
 		/* Why do we check for len < 32768:
 		 * If we won't, we would practically read 2 EOFs but
 		 * the first one has already popped the block_filter
@@ -309,7 +308,7 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 		if( len == -1 )
 		    break;
 		if( len < 32768 )
-		    eof = 1;
+		    eof_seen = 1;
 		if( mfx->md )
 		    gcry_md_write ( mfx->md, buffer, len );
 		if( fp )
