@@ -1,6 +1,6 @@
 /* command.c - SCdaemon command handler
  * Copyright (C) 2001, 2002, 2003, 2004, 2005,
- *               2007, 2008  Free Software Foundation, Inc.
+ *               2007, 2008, 2009  Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -112,6 +112,10 @@ struct server_local_s
      continue operation. */
   int card_removed;        
 
+  /* Flag indicating that the application context needs to be released
+     at the next opportunity.  */
+  int app_ctx_marked_for_release;
+
   /* A disconnect command has been sent.  */
   int disconnect_allowed;
 
@@ -165,7 +169,7 @@ initialize_module_command (void)
 
 
 /* Update the CARD_REMOVED element of all sessions using the reader
-   given by SLOT to VALUE  */
+   given by SLOT to VALUE.  */
 static void
 update_card_removed (int slot, int value)
 {
@@ -184,7 +188,7 @@ update_card_removed (int slot, int value)
 
 
 
-/* Check whether the option NAME appears in LINE */
+/* Check whether the option NAME appears in LINE.  Returns 1 or 0. */
 static int
 has_option (const char *line, const char *name)
 {
@@ -198,7 +202,7 @@ has_option (const char *line, const char *name)
 /* Same as has_option but does only test for the name of the option
    and ignores an argument, i.e. with NAME being "--hash" it would
    return a pointer for "--hash" as well as for "--hash=foo".  If
-   thhere is no such option NULL is returned.  The pointer returned
+   there is no such option NULL is returned.  The pointer returned
    points right behind the option name, this may be an equal sign, Nul
    or a space.  */
 static const char *
@@ -274,11 +278,24 @@ do_reset (ctrl_t ctrl, int send_reset)
   if (!(slot == -1 || (slot >= 0 && slot < DIM(slot_table))))
     BUG ();
 
-  /* If there is an active application, release it. */
+  /* If there is an active application, release it.  Tell all other
+     sessions using the same application to release the
+     application.  */
   if (ctrl->app_ctx)
     {
       release_application (ctrl->app_ctx);
       ctrl->app_ctx = NULL;
+      if (send_reset)
+        {
+          struct server_local_s *sl;
+          
+          for (sl=session_list; sl; sl = sl->next_session)
+            if (sl->ctrl_backlink
+                && sl->ctrl_backlink->reader_slot == slot)
+              {
+                sl->app_ctx_marked_for_release = 1;
+              }
+        }
     }
 
   /* If we want a real reset for the card, send the reset APDU and
@@ -397,14 +414,23 @@ open_card (ctrl_t ctrl, const char *apptype)
   if ( IS_LOCKED (ctrl) )
     return gpg_error (GPG_ERR_LOCKED);
 
-  if (ctrl->app_ctx)
+  /* If the application has been marked for release do it now.  We
+     can't do it immediately in do_reset because the application may
+     still be in use.  */
+  if (ctrl->server_local->app_ctx_marked_for_release)
     {
-      /* Already initialized for one specific application.  Need to
-         check that the client didn't requested a specific application
-         different from the one in use. */
-      return check_application_conflict (ctrl, apptype);
+      ctrl->server_local->app_ctx_marked_for_release = 0;
+      release_application (ctrl->app_ctx);
+      ctrl->app_ctx = NULL;
     }
 
+  /* If we are already initialized for one specific application we
+     need to check that the client didn't requested a specific
+     application different from the one in use before we continue. */
+  if (ctrl->app_ctx)
+    return check_application_conflict (ctrl, apptype);
+
+  /* Setup the slot and select the application.  */
   if (ctrl->reader_slot != -1)
     slot = ctrl->reader_slot;
   else
@@ -436,41 +462,6 @@ open_card (ctrl_t ctrl, const char *apptype)
 }
 
 
-/* Do the percent and plus/space unescaping in place and return the
-   length of the valid buffer. */
-static size_t
-percent_plus_unescape (unsigned char *string)
-{
-  unsigned char *p = string;
-  size_t n = 0;
-
-  while (*string)
-    {
-      if (*string == '%' && string[1] && string[2])
-        { 
-          string++;
-          *p++ = xtoi_2 (string);
-          n++;
-          string+= 2;
-        }
-      else if (*string == '+')
-        {
-          *p++ = ' ';
-          n++;
-          string++;
-        }
-      else
-        {
-          *p++ = *string++;
-          n++;
-        }
-    }
-
-  return n;
-}
-
-
-
 /* SERIALNO [APPTYPE] 
 
    Return the serial number of the card using a status reponse.  This
@@ -528,7 +519,7 @@ cmd_serialno (assuan_context_t ctx, char *line)
 
 
 
-/* LEARN [--force]
+/* LEARN [--force] [--keypairinfo]
 
    Learn all useful information of the currently inserted card.  When
    used without the force options, the command might do an INQUIRE
@@ -538,8 +529,13 @@ cmd_serialno (assuan_context_t ctx, char *line)
 
    The client should just send an "END" if the processing should go on
    or a "CANCEL" to force the function to terminate with a Cancel
-   error message.  The response of this command is a list of status
-   lines formatted as this:
+   error message.  
+
+   With the option --keypairinfo only KEYPARIINFO lstatus lines are
+   returned.
+
+   The response of this command is a list of status lines formatted as
+   this:
 
      S APPTYPE <apptype>
 
@@ -565,7 +561,8 @@ cmd_serialno (assuan_context_t ctx, char *line)
       100 := Regular X.509 cert
       101 := Trusted X.509 cert
       102 := Useful X.509 cert
-      110 := Root CA cert (e.g. DINSIG)
+      110 := Root CA cert in a special format (e.g. DINSIG)
+      111 := Root CA cert as standard X509 cert.
 
    For certain cards, more information will be returned:
 
@@ -589,13 +586,14 @@ cmd_serialno (assuan_context_t ctx, char *line)
 
    The URL to be used for locating the entire public key.
      
-   Note, that this function may be even be used on a locked card.
+   Note, that this function may even be used on a locked card.
 */
 static int
 cmd_learn (assuan_context_t ctx, char *line)
 {
   ctrl_t ctrl = assuan_get_pointer (ctx);
   int rc = 0;
+  int only_keypairinfo = has_option (line, "--keypairinfo");
 
   if ((rc = open_card (ctrl, NULL)))
     return rc;
@@ -604,51 +602,53 @@ cmd_learn (assuan_context_t ctx, char *line)
      the card using a serial number and inquiring the client with
      that. The client may choose to cancel the operation if he already
      knows about this card */
-  {
-    char *serial_and_stamp;
-    char *serial;
-    time_t stamp;
-
-    rc = app_get_serial_and_stamp (ctrl->app_ctx, &serial, &stamp);
-    if (rc)
-      return rc;
-    rc = estream_asprintf (&serial_and_stamp, "%s %lu", serial, (unsigned long)stamp);
-    xfree (serial);
-    if (rc < 0)
-      return out_of_core ();
-    rc = 0;
-    assuan_write_status (ctx, "SERIALNO", serial_and_stamp);
-
-    if (!has_option (line, "--force"))
-      {
-        char *command;
-
-        rc = estream_asprintf (&command, "KNOWNCARDP %s", serial_and_stamp);
-        if (rc < 0)
-          {
-            xfree (serial_and_stamp);
-            return out_of_core ();
-          }
-        rc = 0;
-        rc = assuan_inquire (ctx, command, NULL, NULL, 0); 
-        xfree (command);
-        if (rc)
-          {
-            if (gpg_err_code (rc) != GPG_ERR_ASS_CANCELED)
-              log_error ("inquire KNOWNCARDP failed: %s\n",
-                         gpg_strerror (rc));
-            xfree (serial_and_stamp);
-            return rc; 
-          }
-        /* not canceled, so we have to proceeed */
-      }
-    xfree (serial_and_stamp);
-  }
-
+  if (!only_keypairinfo)
+    {
+      char *serial_and_stamp;
+      char *serial;
+      time_t stamp;
+      
+      rc = app_get_serial_and_stamp (ctrl->app_ctx, &serial, &stamp);
+      if (rc)
+        return rc;
+      rc = estream_asprintf (&serial_and_stamp, "%s %lu",
+                             serial, (unsigned long)stamp);
+      xfree (serial);
+      if (rc < 0)
+        return out_of_core ();
+      rc = 0;
+      assuan_write_status (ctx, "SERIALNO", serial_and_stamp);
+      
+      if (!has_option (line, "--force"))
+        {
+          char *command;
+          
+          rc = estream_asprintf (&command, "KNOWNCARDP %s", serial_and_stamp);
+          if (rc < 0)
+            {
+              xfree (serial_and_stamp);
+              return out_of_core ();
+            }
+          rc = 0;
+          rc = assuan_inquire (ctx, command, NULL, NULL, 0); 
+          xfree (command);
+          if (rc)
+            {
+              if (gpg_err_code (rc) != GPG_ERR_ASS_CANCELED)
+                log_error ("inquire KNOWNCARDP failed: %s\n",
+                           gpg_strerror (rc));
+              xfree (serial_and_stamp);
+              return rc; 
+            }
+          /* Not canceled, so we have to proceeed.  */
+        }
+      xfree (serial_and_stamp);
+    }
+  
   /* Let the application print out its collection of useful status
      information. */
   if (!rc)
-    rc = app_write_learn_status (ctrl->app_ctx, ctrl);
+    rc = app_write_learn_status (ctrl->app_ctx, ctrl, only_keypairinfo);
 
   TEST_CARD_REMOVAL (ctrl, rc);
   return rc;
@@ -871,7 +871,7 @@ pin_cb (void *opaque, const char *info, char **retstr)
 }
 
 
-/* PKSIGN [--hash=[rmd160|sha1|md5]] <hexified_id>
+/* PKSIGN [--hash=[rmd160|sha{1,224,256,384,512}|md5]] <hexified_id>
 
    The --hash option is optional; the default is SHA1.
 
@@ -890,6 +890,14 @@ cmd_pksign (assuan_context_t ctx, char *line)
     hash_algo = GCRY_MD_RMD160;
   else if (has_option (line, "--hash=sha1"))
     hash_algo = GCRY_MD_SHA1;
+  else if (has_option (line, "--hash=sha224"))
+    hash_algo = GCRY_MD_SHA224;
+  else if (has_option (line, "--hash=sha256"))
+    hash_algo = GCRY_MD_SHA256;
+  else if (has_option (line, "--hash=sha384"))
+    hash_algo = GCRY_MD_SHA384;
+  else if (has_option (line, "--hash=sha512"))
+    hash_algo = GCRY_MD_SHA512;
   else if (has_option (line, "--hash=md5"))
     hash_algo = GCRY_MD_MD5;
   else if (!strstr (line, "--"))
@@ -921,7 +929,7 @@ cmd_pksign (assuan_context_t ctx, char *line)
   xfree (keyidstr);
   if (rc)
     {
-      log_error ("card_sign failed: %s\n", gpg_strerror (rc));
+      log_error ("app_sign failed: %s\n", gpg_strerror (rc));
     }
   else
     {
@@ -971,7 +979,7 @@ cmd_pkauth (assuan_context_t ctx, char *line)
   xfree (keyidstr);
   if (rc)
     {
-      log_error ("app_auth_sign failed: %s\n", gpg_strerror (rc));
+      log_error ("app_auth failed: %s\n", gpg_strerror (rc));
     }
   else
     {
@@ -1015,7 +1023,7 @@ cmd_pkdecrypt (assuan_context_t ctx, char *line)
   xfree (keyidstr);
   if (rc)
     {
-      log_error ("card_create_signature failed: %s\n", gpg_strerror (rc));
+      log_error ("app_decipher failed: %s\n", gpg_strerror (rc));
     }
   else
     {
@@ -1110,7 +1118,7 @@ cmd_setattr (assuan_context_t ctx, char *orig_line)
       *line++ = 0;
   while (spacep (line))
     line++;
-  nbytes = percent_plus_unescape ((unsigned char*)line);
+  nbytes = percent_plus_unescape_inplace (line, 0);
 
   rc = app_setattr (ctrl->app_ctx, keyword, pin_cb, ctx,
                     (const unsigned char*)line, nbytes);
@@ -1370,9 +1378,10 @@ cmd_random (assuan_context_t ctx, char *line)
 
 /* PASSWD [--reset] [--nullpin] <chvno>
   
-   Change the PIN or reset the retry counter of the card holder
-   verfication vector CHVNO.  The option --nullpin is used for TCOS
-   cards to set the initial PIN. */
+   Change the PIN or, if --reset is given, reset the retry counter of
+   the card holder verfication vector CHVNO.  The option --nullpin is
+   used for TCOS cards to set the initial PIN.  The format of CHVNO
+   depends on the card application.  */
 static int
 cmd_passwd (assuan_context_t ctx, char *line)
 {
@@ -1435,13 +1444,27 @@ cmd_passwd (assuan_context_t ctx, char *line)
       literal string "[CHV3]": In this case the Admin PIN is checked
       if and only if the retry counter is still at 3.
 
+   For Netkey:
+
+      Any of the valid PIN Ids may be used.  These are the strings:
+
+        PW1.CH       - Global password 1
+        PW2.CH       - Global password 2
+        PW1.CH.SIG   - SigG password 1
+        PW2.CH.SIG   - SigG password 2
+
+      For a definitive list, see the implementation in app-nks.c.
+      Note that we call a PW2.* PIN a "PUK" despite that since TCOS
+      3.0 they are technically alternative PINs used to mutally
+      unblock each other.
+
  */
 static int
 cmd_checkpin (assuan_context_t ctx, char *line)
 {
   ctrl_t ctrl = assuan_get_pointer (ctx);
   int rc;
-  char *keyidstr;
+  char *idstr;
 
   if ( IS_LOCKED (ctrl) )
     return gpg_error (GPG_ERR_LOCKED);
@@ -1455,14 +1478,12 @@ cmd_checkpin (assuan_context_t ctx, char *line)
   /* We have to use a copy of the key ID because the function may use
      the pin_cb which in turn uses the assuan line buffer and thus
      overwriting the original line with the keyid. */
-  keyidstr = xtrystrdup (line);
-  if (!keyidstr)
+  idstr = xtrystrdup (line);
+  if (!idstr)
     return out_of_core ();
   
-  rc = app_check_pin (ctrl->app_ctx,
-                      keyidstr,
-                      pin_cb, ctx);
-  xfree (keyidstr);
+  rc = app_check_pin (ctrl->app_ctx, idstr, pin_cb, ctx);
+  xfree (idstr);
   if (rc)
     log_error ("app_check_pin failed: %s\n", gpg_strerror (rc));
 
@@ -1566,7 +1587,7 @@ cmd_unlock (assuan_context_t ctx, char *line)
    deny_admin  - Returns OK if admin commands are not allowed or
                  GPG_ERR_GENERAL if admin commands are allowed.
 
-   app_list    - Return a list of supported applciations.  One
+   app_list    - Return a list of supported applications.  One
                  application per line, fields delimited by colons,
                  first field is the name.
 */
@@ -1701,7 +1722,7 @@ cmd_disconnect (assuan_context_t ctx, char *line)
 
 
 
-/* APDU [--atr] [--more] [hexstring]
+/* APDU [--atr] [--more] [--exlen[=N]] [hexstring]
 
    Send an APDU to the current reader.  This command bypasses the high
    level functions and sends the data directly to the card.  HEXSTRING
@@ -1714,8 +1735,11 @@ cmd_disconnect (assuan_context_t ctx, char *line)
      S CARD-ATR 3BFA1300FF813180450031C173C00100009000B1
 
    Using the option --more handles the card status word MORE_DATA
-   (61xx) and concatenate all reponses to one block.
+   (61xx) and concatenates all reponses to one block.
 
+   Using the option "--exlen" the returned APDU may use extended
+   length up to N bytes.  If N is not given a default value is used
+   (currently 4096).
  */
 static int
 cmd_apdu (assuan_context_t ctx, char *line)
@@ -1726,10 +1750,22 @@ cmd_apdu (assuan_context_t ctx, char *line)
   size_t apdulen;
   int with_atr;
   int handle_more;
+  const char *s;
+  size_t exlen;
 
   with_atr = has_option (line, "--atr");
   handle_more = has_option (line, "--more");
 
+  if ((s=has_option_name (line, "--exlen")))
+    {
+      if (*s == '=')
+        exlen = strtoul (s+1, NULL, 0);
+      else
+        exlen = 4096;
+    }
+  else
+    exlen = 0;
+
   line = skip_options (line);
 
   if ( IS_LOCKED (ctrl) )
@@ -1766,7 +1802,8 @@ cmd_apdu (assuan_context_t ctx, char *line)
       unsigned char *result = NULL;
       size_t resultlen;
 
-      rc = apdu_send_direct (ctrl->reader_slot, apdu, apdulen, handle_more,
+      rc = apdu_send_direct (ctrl->reader_slot, exlen,
+                             apdu, apdulen, handle_more,
                              &result, &resultlen);
       if (rc)
         log_error ("apdu_send_direct failed: %s\n", gpg_strerror (rc));
@@ -2005,6 +2042,18 @@ send_status_info (ctrl_t ctrl, const char *keyword, ...)
 }
 
 
+/* Send a ready formatted status line via assuan.  */
+void
+send_status_direct (ctrl_t ctrl, const char *keyword, const char *args)
+{
+  assuan_context_t ctx = ctrl->server_local->assuan_ctx;
+
+  if (strchr (args, '\n'))
+    log_error ("error: LF detected in status line - not sending\n");
+  else
+    assuan_write_status (ctx, keyword, args);
+}
+
 
 /* Helper to send the clients a status change notification.  */
 static void
