@@ -16,7 +16,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
- * $Id: iso7816.h 4912 2009-01-08 19:56:30Z wk $
+ * $Id: iso7816.h 5043 2009-06-17 09:45:50Z wk $
  */
 
 #ifndef ISO7816_H
@@ -84,7 +84,7 @@ gpg_error_t iso7816_reset_retry_counter_kp (int slot, int chvno,
 gpg_error_t iso7816_reset_retry_counter_with_rc (int slot, int chvno,
                                                  const char *data,
                                                  size_t datalen);
-gpg_error_t iso7816_get_data (int slot, int tag,
+gpg_error_t iso7816_get_data (int slot, int extended_mode, int tag,
                               unsigned char **result, size_t *resultlen);
 gpg_error_t iso7816_put_data (int slot, int extended_mode, int tag,
                               const unsigned char *data, size_t datalen);
@@ -96,19 +96,21 @@ gpg_error_t iso7816_manage_security_env (int slot, int p1, int p2,
 gpg_error_t iso7816_compute_ds (int slot,
                                 const unsigned char *data, size_t datalen,
                                 unsigned char **result, size_t *resultlen);
-gpg_error_t iso7816_decipher (int slot,
+gpg_error_t iso7816_decipher (int slot, int extended_mode,
                               const unsigned char *data, size_t datalen,
                               int padind,
                               unsigned char **result, size_t *resultlen);
 gpg_error_t iso7816_internal_authenticate (int slot,
                                    const unsigned char *data, size_t datalen,
                                    unsigned char **result, size_t *resultlen);
-gpg_error_t iso7816_generate_keypair (int slot,
-                                   const unsigned char *data, size_t datalen,
-                                   unsigned char **result, size_t *resultlen);
-gpg_error_t iso7816_read_public_key (int slot,
-                                   const unsigned char *data, size_t datalen,
-                                   unsigned char **result, size_t *resultlen);
+gpg_error_t iso7816_generate_keypair (int slot, int extended_mode,
+                                    const unsigned char *data, size_t datalen,
+                                    int le,
+                                    unsigned char **result, size_t *resultlen);
+gpg_error_t iso7816_read_public_key (int slot, int extended_mode,
+                                    const unsigned char *data, size_t datalen,
+                                    int le,
+                                    unsigned char **result, size_t *resultlen);
 gpg_error_t iso7816_get_challenge (int slot,
                                    int length, unsigned char *buffer);
 
